package net.tomeko.rtpstreamer;

import android.content.Context;
import android.content.SharedPreferences;
import android.net.rtp.AudioGroup;
import android.net.rtp.AudioStream;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;
import android.util.Log;
import android.media.AudioManager;
import android.net.rtp.*;

import java.net.*;

public class MainActivity extends AppCompatActivity {

    EditText edIp;
    EditText edPort;
    TextView textViewStatus;

    private static AudioStream myAudioStream;
    private static AudioGroup myAudioGroup;
    private static AudioManager myAudioManager;

    private enum StreamerState {
        STREAMER_STATE_IDLE,
        STREAMER_STATE_STREAMING,
        STREAMER_STATE_STOPPED
    };
    private static StreamerState streamerState = StreamerState.STREAMER_STATE_IDLE;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        SharedPreferences prefs = this.getSharedPreferences("net.tomeko.rtpstreamer", Context.MODE_PRIVATE);
        String ip = prefs.getString("net.tomeko.rtpstreamer.ip", "192.168.1.170");
        String port = prefs.getString("net.tomeko.rtpstreamer.port", "7777");

        edIp = (EditText) findViewById(R.id.edIp);
        edIp.setText(ip);
        edIp.setSelection(edIp.getText().length());
        edPort = (EditText) findViewById(R.id.edPort);
        edPort.setText(port);
        textViewStatus = (TextView) findViewById(R.id.textViewStatus);
        updateState(streamerState);
    }

    public void onClickStart(View v) {
        if (myAudioStream != null) {
            return;
        }
        java.net.InetAddress address;
        try {
            address = InetAddress.getByName(edIp.getText().toString());
        } catch (UnknownHostException e) {
            return;
        } catch (android.os.NetworkOnMainThreadException e) {
            return;
        }
        Integer port = Integer.parseInt(edPort.getText().toString());

        SharedPreferences prefs = this.getSharedPreferences("net.tomeko.rtpstreamer", Context.MODE_PRIVATE);
        prefs.edit().putString("net.tomeko.rtpstreamer.ip", edIp.getText().toString()).apply();
        prefs.edit().putString("net.tomeko.rtpstreamer.port", edPort.getText().toString()).apply();

        Locks.acquireMulticastLock(this);
        Locks.acquireWifiHighPerfLock(this);

        myAudioManager = (AudioManager) getSystemService(Context.AUDIO_SERVICE);
        myAudioGroup = new AudioGroup();
        myAudioGroup.setMode(AudioGroup.MODE_ECHO_SUPPRESSION);

        try {
            myAudioStream = new AudioStream(InetAddress.getByName("239.255.10.10"));
        } catch (SocketException ex) {
            Log.d("Error", "Cannot create audio stream");
        } catch (UnknownHostException ex) {
            Log.d("Error", "Cannot create audio stream");
        }

        myAudioStream.setCodec(AudioCodec.PCMA);

        myAudioStream.setMode(RtpStream.MODE_SEND_ONLY);

        myAudioStream.associate(address, port);
        myAudioStream.join(myAudioGroup);

        myAudioManager.setMode(AudioManager.MODE_IN_COMMUNICATION);

        myAudioManager.setSpeakerphoneOn(false);
        myAudioManager.setMicrophoneMute(false);
        updateState(StreamerState.STREAMER_STATE_STREAMING);
    }

    public void onClickStop(View v) {
        if (myAudioStream != null) {
            myAudioStream.join(null);
            myAudioGroup.setMode(AudioGroup.MODE_ON_HOLD);
            myAudioStream.release();
            myAudioStream = null;
            myAudioGroup = null;
            updateState(StreamerState.STREAMER_STATE_STOPPED);
        }
    }

    private void updateState(StreamerState state) {
        streamerState = state;
        switch (streamerState) {
            case STREAMER_STATE_IDLE:
                textViewStatus.setText("Idle");
                break;
            case STREAMER_STATE_STOPPED:
                textViewStatus.setText("Stopped");
                break;
            case STREAMER_STATE_STREAMING:
                textViewStatus.setText("Streaming...");
                break;
        }
    }
}
